# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['saw_client']

package_data = \
{'': ['*']}

install_requires = \
['BitVector>=3.4.9,<4.0.0',
 'argo-client==0.0.4',
 'cryptol==2.11.0',
 'requests>=2.25.1,<3.0.0']

setup_kwargs = {
    'name': 'saw-client',
    'version': '0.8.1',
    'description': 'SAW client for the SAW 0.8 RPC server',
    'long_description': '# SAW Python Client\n\nIn-development Python client for SAW. Currently tested on Linux and MacOS -- at present there may be some minor issues running the Python Client in a Windows environment that needs addressed (e.g., some Python process management methods are not cross-OS-compatible).\n\nThis SAW client depends on the [saw-remote-api](https://github.com/GaloisInc/saw-script/tree/master/saw-remote-api) server.\n\n# TL;DR Steps to running SAW Python scripts\n\n1. Clone the repo \n```\ngit clone https://github.com/GaloisInc/saw-script.git\n```\n2. Enter the repo\n```\ncd saw-script\n```\n3. Initialize git submodules \n```\ngit submodule update --init\n```\n4. Navigate to the python client\n```\ncd saw-remote-api/python\n```\n5. Install and setup some version of the `saw-remote-api` server and update any\n   relevant environment variables as needed (see `saw_client.connect()` documentation\n   for the various ways a server can be connected to).\n   E.g., here is how the docker image of the server might be used:\n```\n$ docker run --name=saw-remote-api -d \\\n  -v $PWD/tests/saw/test-files:/home/saw/tests/saw/test-files \\\n  -p 8080:8080 \\\n  galoisinc/saw-remote-api:nightly\n$ export SAW_SERVER_URL="http://localhost:8080/"\n```\n6. Install the Python client (requires Python v3.8 or newer -- we recommend using [`poetry`](https://python-poetry.org/docs/#installation) to install the package):\n```\n$ poetry install \n```\n7. Run tests or individual scripts:\n```\n$ poetry run python -m unittest discover tests/saw\n$ poetry run python tests/saw/test_salsa20.py\n```\n\n(We\'re aware the tests currently emit some `ResourceWarning`s regarding\nsubprocesses when run via `unittest` even though they pass and successfully\nverify the goals. It\'s on our to-do list.)\n\n# Python Client Installation (via Poetry)\n\nFirst, clone the repository and submodules.\n\n```\n$ git clone https://github.com/GaloisInc/saw-script.git\n$ cd saw-script\n$ git submodule update --init\n```\n\nThen, use [`poetry`](https://python-poetry.org/docs/#installation) to install\nthe python client from the `saw-remote-api/python` directory:\n\n```\n$ cd saw-remote-api/python\n$ poetry install\n```\n\n# SAW server\n\nTo run the verification scripts a `saw-remote-api` server must be available,\neither as a local executable or running in docker image listening on a port.\n\n## Connecting with a server in a script\n\nConnecting to a server in a Python script is accomplished via the `saw_client.connect`\nmethod. Its accompanying Python doc strings describe the various ways it can be\nused. Below is a brief summary:\n\n`saw_client.connect()`, when provided no arguments, will attempt the following in order:\n\n1. If the environment variable ``SAW_SERVER`` is set and refers to an\n   executable, it is assumed to be a `saw-remote-api` executable and will be\n   used for the duration of the script.\n2. If the environment variable ``SAW_SERVER_URL`` is set, it is assumed to be\n   the URL for a running SAW server in ``http`` mode and will be connected to.\n   (N.B., this can be a local server or a server running in a docker image.)\n3. If an executable ``saw-remote-api`` is available on the ``PATH`` it is\n    assumed to be a SAW server and will be used for the duration of the script.\n\nAdditional arguments and options are documented with the function.\n\nNotable, the `reset_server` keyword can be used to connect to a running server\nand reset it, ensuring states from previous scrips have been cleared. E.g.,\n`saw_client.connect(reset_server=True)`.\n\n\n## Acquiring a SAW Server\n\nThere are several ways a server executable can be obtained.\n\n### Server executables\n\nAn executable of the server is included in the SAW release/nightly tarballs.\n\nIf using a SAW [release](https://github.com/GaloisInc/saw-script/releases), it\nis recommended to use `v0.8` or greater if possible. (`v0.7` _does_ include the\nserver and may work for individual use cases, but running repeated scripts\nagainst the same persistent server will not work as intended.)\n\nNightly server builds can be found as `Artifacts` of the [Nightly\nBuilds](https://github.com/GaloisInc/saw-script/actions/workflows/nightly.yml)\ngithub action. I.e., go to the `Nightly Builds` Github Action, click on a\nsuccessful build, scroll to the bottom and under `Artifacts` a Linux, Windows,\nand MacOS tarball will be listed. (Apologies... we need to make these easier to\nfind...)\n\n### Server docker images\n\nDocker images for the SAW server are currently uploaded to\n[DockerHub](https://hub.docker.com/r/galoisinc/saw-remote-api).\n\nThese images are set up to run as HTTP `saw-remote-api` servers, e.g.:\n\n```\ndocker run --name=saw-remote-api -d \\\n  -p 8080:8080 \\\n  galoisinc/saw-remote-api:TAG\n```\n\n(where `TAG` is either `latest` or `nightly`) will launch a server listening at\n`http://localhost:8080/`. (As of March 2020, `nightly` is recommended, as the\nserver in the last official release (i.e., the one accompanying SAW `v0.7`)\ncontains some non-trivial bugs that greatly limit its utility.)\n\nThe `-v` option to `docker run` can be used to load files into the docker\nserver\'s working directory so they can be loaded into the server at the request\nof python scripts. E.g., `-v PATH_TO_FILES:/home/saw/files/` will upload the\ncontents of the host machine\'s directory `PATH_TO_FILES` to the\n`/home/saw/files/` directory in the docker container, which corresponds to the\nrelative path `files/` for the SAW server. (If desired, it can be useful to\nplace files in a location in the Docker container such that the same relative\npaths in scripts refer to the same files in the host machine and docker\ncontainer.)\n\n### Building from Source\n\nIf this repository is checked out and the build directions are successfully run,\n`cabal v2-exec which saw-remote-api` should indicate where the server executable has\nbeen stored by `cabal`.\n\nAlternatively `cabal v2-install saw-remote-api` should install the server\nexecutable into the user\'s `~/.cabal/bin` directory (or similar), which (if\nconfigured properly) should then appear as `saw-remote-api` in a user\'s `PATH`.\n\n\n# Running Python SAW verification scripts\n\nOnce the server is setup and any path variables are setup as desired, the\nPython (>= v3.8) client can be installed using\n[`poetry`](https://python-poetry.org/docs/#installation) as follows:\n\n```\n$ cd saw-remote-api/python\n$ poetry install\n```\n\nThen the tests or individual scripts can be run as follows:\n```\n$ poetry run python -m unittest discover tests/saw\n$ poetry run python tests/saw/test_salsa20.py\n```\n\nIf leveraging environment variables is undesirable, the scripts themselves can\nspecify a command to launch the server, e.g.:\n\n```\nsaw_client.connect(COMMAND)\n```\n\nwhere `COMMAND` is a command to launch a new SAW server in socket mode.\n\nOr a server URL can be specified directly in the script, e.g.:\n\n```\nsaw_client.connect(url=URL)\n```\n\nwhere `URL` is a URL for a running SAW server in HTTP mode.\n\n## Running Verification Scripts from a clean state\n\nTo ensure any previous server state is cleared when running a SAW Python script\nagainst a persistent server (e.g., one running in HTTP mode in a different process),\nthe `reset_server` keyword can be passed to `saw_client.connect()`. E.g.,\n\n```\nsaw_client.connect(url="http://localhost:8080/", reset_server=True)\n```\n\nwill connect to a SAW server running at `http://localhost:8080/` and will\nguarantee any previous state on the server is cleared.\n',
    'author': 'Andrew Kent',
    'author_email': 'andrew@galois.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
